<?php

namespace Scau\UnifiedLogin;

use Scau\Common\Utils;
use Scau\Core\ScauBase;
use Scau\Exception\UnifiedLoginException;

/**
 * 授权类
 * @package Scau\UnifiedLogin
 */
class Authorization extends ScauBase
{
    /**
     * Authorization constructor.
     * @param string $appId 应用ID
     * @param string $appSecret 应用密钥
     * @param string $redirectUri 回调地址，可用url_encode()编码
     * @param array $config 配置
     */
    public function __construct(string $appId, string $appSecret, string $redirectUri, array $config = [])
    {
        $this->appId = $appId;
        $this->appSecret = $appSecret;
        $this->redirectUri = $redirectUri;
        $this->config = array_merge($this->getDefaultConfig(), $config);
    }

    public function getOauthUrl(string $state = ''): string
    {
        return Utils::urlencodeParameters(
            $this->getOauthDomain() . "/oauth/grant",
            [
                'app_id' => $this->getAppId(),
                'redirect_uri' => $this->getRedirectUri(),
                'response_type' => 'code',
                'state' => $state
            ]
        );
    }

    /**
     * @param string $code
     * @return array
     * @throws UnifiedLoginException
     */
    public function credential(string $code): array
    {
        $this->getAccessToken();
        $openid = $this->getOpenid($code);
        $userInfo = $this->getUserInfo($openid);
        $userInfo['openid'] = $openid;// 加上openid返回
        return $userInfo;
    }

    /**
     * @return string
     * @throws UnifiedLoginException
     */
    public function getAccessToken(): string
    {
        $tokenFilePath = $this->getAccessTokenFilePath();
        $accessToken = '';
        if (file_exists($tokenFilePath)) {
            $tokenData = json_decode(file_get_contents($tokenFilePath), true);
            if ($tokenData && $tokenData['access_token'] && $tokenData['expire_time'] > time() - 600) {
                $accessToken = $tokenData['access_token'];
            }
        }

        if (!$accessToken) {
            $url = Utils::urlencodeParameters(
                $this->getApiDomain() . "/applications/getAccessToken",
                [
                    'app_id' => $this->getAppId(),
                    'app_secret' => $this->getAppSecret()
                ]
            );
            $data = Utils::httpCurl($url);
            $data = json_decode($data, true);
            if (!$data) {
                throw new UnifiedLoginException('An unexpected exception occurred!');
            }
            if ($data['code']) {
                throw new UnifiedLoginException($data['msg']);
            }

            $accessToken = $data['data']['access_token'];
            $tokenData = $data['data'];
            $tokenData['expire_time'] = $tokenData['create_time'] + $tokenData['expire_in'];
            unset($tokenData['create_time'], $tokenData['expire_in']);
            file_put_contents($tokenFilePath, json_encode($tokenData));
        }

        $this->accessToken = $accessToken;

        return $accessToken;
    }

    /**
     * @param string $code
     * @return string
     * @throws UnifiedLoginException
     */
    public function getOpenid(string $code): string
    {
        $url = Utils::urlencodeParameters(
            $this->getApiDomain() . "/user/getOpenid",
            [
                'app_id' => $this->getAppId(),
                'access_token' => $this->accessToken,
                'code' => $code
            ]
        );
        $data = Utils::httpCurl($url);
        $data = json_decode($data, true);
        if (!$data) {
            throw new UnifiedLoginException('An unexpected exception occurred!');
        }
        if ($data['code']) {
            throw new UnifiedLoginException($data['msg']);
        }

        return $data['data']['openid'];
    }

    /**
     * @param string $openid
     * @return array
     * @throws UnifiedLoginException
     */
    public function getUserInfo(string $openid): array
    {
        $url = Utils::urlencodeParameters(
            $this->getApiDomain() . "/user/getInfo",
            [
                'app_id' => $this->getAppId(),
                'access_token' => $this->accessToken,
                'openid' => $openid
            ]
        );
        $data = Utils::httpCurl($url);
        $data = json_decode($data, true);
        if (!$data) {
            throw new UnifiedLoginException('An unexpected exception occurred!');
        }
        if ($data['code']) {
            throw new UnifiedLoginException($data['msg']);
        }

        return $data['data'];
    }
}